@@ -1,4 +1,23 @@
 # -*- coding: UTF-8 -*-
+##############################################################################
+#
+#    OERPLib
+#    Copyright (C) 2011-2013 Sébastien Alix.
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU Lesser General Public License as published
+#    by the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU Lesser General Public License for more details.
+#
+#    You should have received a copy of the GNU Lesser General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+##############################################################################
 """This module contains the ``OERP`` class which manage the interaction with
 the `OpenERP` server.
 """
@@ -8,11 +27,9 @@ import zlib
 import tempfile
 import time
 
-from oerplib import config
-from oerplib import rpc
-from oerplib import error
-from oerplib.tools import detect_version
-from oerplib.service import common, db, wizard, osv
+from oerplib import rpc, error, tools
+from oerplib.tools import session
+from oerplib.service import common, db, wizard, osv, inspect
 
 
 class OERP(object):
@@ -40,26 +57,30 @@ class OERP(object):
 
     def __init__(self, server='localhost', database=None, protocol='xmlrpc',
                  port=8069, timeout=120, version=None):
+        if protocol not in ['xmlrpc', 'xmlrpc+ssl', 'netrpc']:
+            txt = ("The protocol '{0}' is not supported by the OERP class. "
+                   "Please choose a protocol among these ones: {1}")
+            txt = txt.format(protocol, ['xmlrpc', 'xmlrpc+ssl', 'netrpc'])
+            raise error.InternalError(txt)
         self._server = server
         self._port = port
         self._protocol = protocol
         self._database = self._database_default = database
         self._uid = None
         self._password = None
+        self._user = None
         self._common = common.Common(self)
         self._db = db.DB(self)
         self._wizard = wizard.Wizard(self)
-        self._version = version or detect_version(
-            server, protocol, port, timeout)
+        self._inspect = inspect.Inspect(self)
         # Instanciate the OpenERP server connector
         try:
-            self._connector = rpc.get_connector(
-                self._server, self._port, self._protocol,
-                timeout, self._version)
+            self._connector = rpc.PROTOCOLS[protocol](
+                self._server, self._port, timeout, version)
         except rpc.error.ConnectorError as exc:
             raise error.InternalError(exc.message)
         # Dictionary of configuration options
-        self._config = config.Config(
+        self._config = tools.Config(
             self,
             {'auto_context': True,
              'timeout': timeout})
@@ -122,6 +143,15 @@ class OERP(object):
         """
         return self._context
 
+    @property
+    def version(self):
+        """The version of the OpenERP server.
+
+        >>> oerp.version
+        '7.0-20131014-231047'
+        """
+        return self._connector.version
+
     server = property(lambda self: self._server,
                       doc="The server name.")
     port = property(lambda self: self._port,
@@ -147,21 +177,12 @@ class OERP(object):
                        The wizard service (``/wizard`` RPC service).
                        See the :class:`oerplib.service.wizard.Wizard` class."""))
 
-    # RPC Connector timeout
-    @property
-    def timeout(self):
-        """.. versionadded:: 0.6
+    inspect = property(lambda self: self._inspect,
+                       doc=(""".. versionadded:: 0.8
 
-        .. deprecated:: 0.7 will be deleted in the next version. Use the
-            :attr:`OERP.config <oerplib.OERP.config>` property instead
-
-        Set the maximum timeout for a RPC request.
-        """
-        return self.config['timeout']
-
-    @timeout.setter
-    def timeout(self, timeout):
-        self.config['timeout'] = timeout
+                       The inspect service (custom service).
+                       See the :class:`oerplib.service.inspect.Inspect`
+                       class."""))
 
     #NOTE: in the past this function was implemented as a decorator for other
     # methods needed to be checked, but Sphinx documentation generator is not
@@ -169,8 +190,7 @@ class OERP(object):
     def _check_logged_user(self):
         """Check if a user is logged. Otherwise, an error is raised."""
         if not self._uid or not self._password:
-            raise error.Error(
-                u"User login required.")
+            raise error.Error(u"User login required.")
 
     def login(self, user='admin', passwd='admin', database=None):
         """Log in as the given `user` with the password `passwd` on the
@@ -183,12 +203,12 @@ class OERP(object):
         u'Administrator'
 
         :return: the user connected as a browsable record
-        :raise: :class:`oerplib.error.RPCError`
+        :raise: :class:`oerplib.error.RPCError`, :class:`oerplib.error.Error`
         """
         # Raise an error if no database was given
         self._database = database or self._database_default
         if not self._database:
-            raise error.Error(u"No database specified")
+            raise error.Error("No database specified")
         # Get the user's ID and generate the corresponding User record
         try:
             user_id = self.common.login(self._database, user, passwd)
@@ -203,7 +223,7 @@ class OERP(object):
                 return self._user
             else:
                 #FIXME: Raise an error?
-                raise error.RPCError(u"Wrong login ID or password")
+                raise error.RPCError("Wrong login ID or password")
 
     # ------------------------- #
     # -- Raw XML-RPC methods -- #
@@ -251,8 +271,7 @@ class OERP(object):
         kwargs = kwargs or {}
         try:
             return self._connector.object.execute_kw(
-                self._database, self._uid,
-                self._password,
+                self._database, self._uid, self._password,
                 model, method, args, kwargs)
         except rpc.error.ConnectorError as exc:
             raise error.RPCError(exc.message, exc.oerp_traceback)
@@ -267,9 +286,9 @@ class OERP(object):
         self._check_logged_user()
         # Execute the workflow query
         try:
-            self._connector.object.exec_workflow(self._database, self._uid,
-                                                 self._password,
-                                                 model, signal, obj_id)
+            self._connector.object.exec_workflow(
+                self._database, self._uid, self._password,
+                model, signal, obj_id)
         except rpc.error.ConnectorError as exc:
             raise error.RPCError(exc.message, exc.oerp_traceback)
 
@@ -313,7 +332,7 @@ class OERP(object):
         }
         try:
             report_id = self._connector.report.report(
-                self._database, self.user.id, self.user.password,
+                self._database, self.user.id, self._password,
                 report_name, obj_ids, data, context)
         except rpc.error.ConnectorError as exc:
             raise error.RPCError(exc.message, exc.oerp_traceback)
@@ -322,7 +341,7 @@ class OERP(object):
         while not state:
             try:
                 pdf_data = self._connector.report.report_get(
-                    self._database, self.user.id, self.user.password,
+                    self._database, self.user.id, self._password,
                     report_id)
             except rpc.error.ConnectorError as exc:
                 raise error.RPCError("Unknown error occurred during the "
@@ -341,7 +360,7 @@ class OERP(object):
         """Print data in a temporary file and return the path of this one."""
         if 'result' not in data:
             raise error.InternalError(
-                u"Invalid data, the operation has been canceled.")
+                "Invalid data, the operation has been canceled.")
         content = base64.decodestring(data['result'])
         if data.get('code') == 'zlib':
             content = zlib.decompress(content)
@@ -427,7 +446,7 @@ class OERP(object):
         """Update `model` records identified by `ids` with the given values
         contained in the `vals` dictionary.
 
-        >>> oerp.write('res.users', [1], {'name': u"Administrator"})
+        >>> oerp.write('res.users', [1], {'name': "Administrator"})
         True
 
         :return: `True`
@@ -469,7 +488,7 @@ class OERP(object):
         :raise: :class:`oerplib.error.RPCError`
         """
         if not isinstance(browse_record, osv.BrowseRecord):
-            raise ValueError(u"An instance of BrowseRecord is required")
+            raise ValueError("An instance of BrowseRecord is required")
         return osv.Model(self, browse_record.__osv__['name'])._write_record(
             browse_record, context)
 
@@ -486,7 +505,7 @@ class OERP(object):
         :raise: :class:`oerplib.error.RPCError`
         """
         if not isinstance(browse_record, osv.BrowseRecord):
-            raise ValueError(u"An instance of BrowseRecord is required")
+            raise ValueError("An instance of BrowseRecord is required")
         return osv.Model(self, browse_record.__osv__['name'])._unlink_record(
             browse_record, context)
 
@@ -522,7 +541,7 @@ class OERP(object):
 
         """
         if not isinstance(browse_record, osv.BrowseRecord):
-            raise ValueError(u"Value is not a browse_record.")
+            raise ValueError("Value is not a browse_record.")
         return browse_record.__osv__['name']
 
     def get(self, model):
@@ -535,4 +554,98 @@ class OERP(object):
         """
         return osv.Model(self, model)
 
+    def save(self, name, rc_file='~/.oerplibrc'):
+        """.. versionadded:: 0.8
+
+        Save the session configuration under the name `name`.
+        These informations are stored in the ``~/.oerplibrc`` file by default.
+
+            >>> import oerplib
+            >>> oerp = oerplib.OERP('localhost', protocol='xmlrpc', port=8069)
+            >>> oerp.login('admin', 'admin', 'db_name')
+            >>> oerp.save('foo')
+
+        Such informations can be loaded with the :func:`oerplib.load` function
+        by returning a pre-configured session of :class:`OERP <oerplib.OERP>`,
+        or with the `oerp` command line tool supplied with `oerplib`.
+        """
+        self._check_logged_user()
+        data = {
+            'type': self.__class__.__name__,
+            'server': self.server,
+            'protocol': self.protocol,
+            'port': self.port,
+            'timeout': self.config['timeout'],
+            'user': self.user.login,
+            'passwd': self._password,
+            'database': self.database,
+        }
+        session.save(name, data, rc_file)
+
+    @classmethod
+    def load(cls, name, rc_file='~/.oerplibrc'):
+        """.. versionadded:: 0.8
+
+        Return a :class:`OERP` session pre-configured and connected
+        with informations identified by `name`:
+
+            >>> import oerplib
+            >>> oerp = oerplib.OERP.load('foo')
+
+        Such informations are stored with the :func:`OERP.save <oerplib.OERP.save>`
+        method.
+        """
+        data = session.get(name, rc_file)
+        if data.get('type') != cls.__name__:
+            raise error.Error(
+                "'{0}' session is not of type '{1}'".format(
+                    name, cls.__name__))
+        oerp = cls(
+            server=data['server'],
+            protocol=data['protocol'],
+            port=data['port'],
+            timeout=data['timeout'],
+        )
+        oerp.login(
+            user=data['user'], passwd=data['passwd'],
+            database=data['database'])
+        return oerp
+
+    @classmethod
+    def list(cls, rc_file='~/.oerplibrc'):
+        """.. versionadded:: 0.8
+
+        Return a list of all sessions available in the
+        `rc_file` file:
+
+            >>> import oerplib
+            >>> oerplib.OERP.list()
+            ['foo', 'bar']
+
+        Then, use the :func:`load` function with the desired session:
+
+            >>> oerp = oerplib.OERP.load('foo')
+        """
+        sessions = session.get_all(rc_file)
+        return [name for name, data in sessions.iteritems()
+                if data.get('type') == cls.__name__]
+        #return session.list(rc_file)
+
+    @classmethod
+    def remove(cls, name, rc_file='~/.oerplibrc'):
+        """.. versionadded:: 0.8
+
+        Remove the session identified by `name` from the `rc_file` file:
+
+            >>> import oerplib
+            >>> oerplib.OERP.remove('foo')
+            True
+        """
+        data = session.get(name, rc_file)
+        if data.get('type') != cls.__name__:
+            raise error.Error(
+                "'{0}' session is not of type '{1}'".format(
+                    name, cls.__name__))
+        return session.remove(name, rc_file)
+
 # vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
