@@ -1,4 +1,23 @@
 # -*- coding: UTF-8 -*-
+##############################################################################
+#
+#    OERPLib
+#    Copyright (C) 2011-2013 Sébastien Alix.
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU Lesser General Public License as published
+#    by the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU Lesser General Public License for more details.
+#
+#    You should have received a copy of the GNU Lesser General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+##############################################################################
 """Provide the :class:`Common` class in order to manage common operations on
 the `OpenERP` server.
 """
@@ -27,7 +46,7 @@ class Common(object):
 
         Method calls are purely dynamic, and the following documentation can be
         wrong if the API of `OpenERP` is changed between versions. Anyway, if
-        you known the API used by the `OpenERP` server for the ``/common`` RPC
+        you know the API used by the `OpenERP` server for the ``/common`` RPC
         service, it will work.
 
     .. method:: Common.login(db, login, password)
@@ -35,12 +54,14 @@ class Common(object):
         >>> oerp.common.login('test_db', 'admin', 'admin_passwd')
         1
 
-        :return: the user's ID
+        :return: the user's ID or `False`
 
     .. method:: Common.authenticate(db, login, password, user_agent_env)
 
-        >>> oerp.common.authenticate('test_db', 'admin', 'admin_passwd', TODO)
-        TODO
+        >>> oerp.common.authenticate('test_db', 'admin', 'admin_passwd', {})
+        1
+
+        :return: the user's ID or `False`
 
     .. method:: Common.version()
 
@@ -90,8 +111,7 @@ class Common(object):
 
     .. method:: Common.set_loglevel(loglevel, logger=None)
 
-        >>> oerp.common.set_loglevel(TODO)
-        True
+        >>> oerp.common.set_loglevel('DEBUG')
 
     .. method:: Common.get_stats()
 
@@ -115,10 +135,17 @@ class Common(object):
         >>> oerp.common.get_os_time()
         (0.01, 0.0, 0.0, 0.0, 17873633.129999999)
 
-    .. method:: Common.get_sql_count()
+    .. method:: Common.get_sqlcount()
 
-        >>> oerp.common.get_sql_count()
-        TODO
+        >>> oerp.common.get_sqlcount()
+
+    .. method:: Common.get_available_updates(super_admin_password, contract_id, contract_password)
+
+        >>> oerp.common.get_available_updates('super_admin_passwd', 'MY_CONTRACT_ID', 'MY_CONTRACT_PASSWORD')
+
+    .. method:: Common.get_migration_scripts(super_admin_password, contract_id, contract_password)
+
+        >>> oerp.common.get_migration_scripts('super_admin_passwd', 'MY_CONTRACT_ID', 'MY_CONTRACT_PASSWORD')
 
     """
     def __init__(self, oerp):
