@@ -269,13 +269,19 @@ class Dependencies(object):
 
     def _fix_fake_root_module(self, module):
         """Fix the fake root `module` by finding its indirect dependencies."""
+        known_paths = []
+
         def find_path(path, mod, common_model):
             """Try to found a path from the module `mod` among all installed
             modules to reach any 'restricted' module.
             """
+            if set(path) not in known_paths:
+                known_paths.append(set(path))
             path.append(mod)
             for depend in self._modules_full[mod]['depends']:
                 path.append(depend)
+                if set(path) in known_paths:
+                    continue
                 if depend in self._modules:
                     if common_model:
                         # Has the 'head' module a common data model with
@@ -287,7 +293,9 @@ class Dependencies(object):
                     else:
                         return True
                 path.pop()
-                return find_path(path, depend, common_model)
+                res = find_path(path, depend, common_model)
+                if res:
+                    return res
             path.pop()
             return False
 
@@ -296,6 +304,8 @@ class Dependencies(object):
         found_ok = find_path(path, module, common_model=True)
         # If not, we try again without the rule of the common model
         if not found_ok:
+            known_paths = []
+            path = []
             found_ok = find_path(path, module, common_model=False)
         # Update the graph by adding required modules to satisfy the
         # indirect dependency
@@ -382,7 +392,7 @@ class Dependencies(object):
                 module_color_title=module_color_title,
                 module_bgcolor_title=module_bgcolor_title,
                 module_bgcolor=self._config['module_bgcolor'],
-                name=module.upper(),
+                name=module.lower(),
                 models=''.join(tpl_models),
                 comment=tpl_comment or '')
             return tpl
