@@ -1,7 +1,27 @@
 # -*- coding: UTF-8 -*-
+##############################################################################
+#
+#    OERPLib
+#    Copyright (C) 2011-2013 Sébastien Alix.
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU Lesser General Public License as published
+#    by the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU Lesser General Public License for more details.
+#
+#    You should have received a copy of the GNU Lesser General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+##############################################################################
 """Provide the :class:`Model` class which allow to access dynamically to all
 methods proposed by an model of the `OpenERP` server."""
 
+import sys  # to check Python version at runtime
 import collections
 
 from oerplib.tools import v
@@ -44,14 +64,6 @@ class Model(object):
         self._name = model
         self._browse_class = self._generate_browse_class()
 
-    def _browse_generator(self, ids, context=None):
-        """Generator used by the
-        :func:`browse <oerplib.OERP.service.osv.Model.browse>` method.
-
-        """
-        for o_id in ids:
-            yield self.browse(o_id, context)
-
     def browse(self, ids, context=None):
         """Browse one or several records (if `ids` is a list of IDs)
         from `model`. The fields and values for such objects are generated
@@ -72,7 +84,11 @@ class Model(object):
 
         """
         if isinstance(ids, list):
-            return self._browse_generator(ids, context)
+            return browse.BrowseRecordIterator(self, ids, context=context)
+            #return browse.BrowseRecordIterator(
+            #    model=self,
+            #    ids=ids,
+            #    context=context)
         else:
             obj = self._browse_class(ids)
             self._refresh(obj, context)
@@ -87,7 +103,9 @@ class Model(object):
         # Retrieve server fields info and generate corresponding local fields
         fields_get = self._oerp.execute(self._name, 'fields_get')
         cls_name = self._name.replace('.', '_')
-        if type(cls_name) == unicode:
+        # Encode the class name for the Python2 'type()' function.
+        # No need to do this for Python3.
+        if type(cls_name) == unicode and sys.version_info < (3,):
             cls_name = cls_name.encode('utf-8')
         cls_fields = {}
         for field_name, field_data in fields_get.items():
@@ -112,10 +130,10 @@ class Model(object):
         """Send values of fields updated to the OpenERP server."""
         obj_data = obj.__data__
         vals = {}
-        for field_name in obj_data['fields_updated']:
+        for field_name in obj_data['updated_values']:
             if field_name in obj_data['raw_data']:
                 field = self._browse_class.__osv__['columns'][field_name]
-                field_value = obj.__data__['values'][field_name]
+                field_value = obj.__data__['updated_values'][field_name]
                 # Many2One fields
                 if isinstance(field, fields.Many2OneField):
                     vals[field_name] = field_value and field_value[0]
@@ -123,7 +141,7 @@ class Model(object):
                 else:
                     vals[field_name] = field_value
         try:
-            if v(self._oerp._version) < v('6.1'):
+            if v(self._oerp.version) < v('6.1'):
                 res = self.write([obj.id], vals, context)
             else:
                 res = self.write([obj.id], vals, context=context)
@@ -153,7 +171,7 @@ class Model(object):
                 obj_data['raw_data'][field_name] = None
         # Fill fields with values of the record
         if obj.id:
-            if v(self._oerp._version) < v('6.1'):
+            if v(self._oerp.version) < v('6.1'):
                 data = self.read([obj.id], basic_fields, context)
                 if data:
                     obj_data['raw_data'].update(data[0])
@@ -167,11 +185,11 @@ class Model(object):
                     obj_data['raw_data'] = False
             if obj_data['raw_data'] is False:
                 raise error.RPCError(
-                    u"There is no '{model}' record with ID {obj_id}.".format(
+                    "There is no '{model}' record with ID {obj_id}.".format(
                         model=obj.__class__.__osv__['name'], obj_id=obj.id))
         # No ID: fields filled with default values
         else:
-            if v(self._oerp._version) < v('6.1'):
+            if v(self._oerp.version) < v('6.1'):
                 default_get = self.default_get(
                     obj.__osv__['columns'].keys(), context)
             else:
@@ -190,18 +208,17 @@ class Model(object):
 
         """
         obj_data = obj.__data__
-        obj_data['fields_updated'] = []
+        obj_data['updated_values'] = {}
         # Load fields and their values
         for field in self._browse_class.__osv__['columns'].values():
             if field.name in obj_data['raw_data']:
                 obj_data['values'][field.name] = \
                     obj_data['raw_data'][field.name]
-                setattr(obj.__class__, field.name,
-                        field)
+                setattr(obj.__class__, field.name, field)
 
     def _unlink_record(self, obj, context=None):
         """Delete the object from the OpenERP server."""
-        if v(self._oerp._version) < v('6.1'):
+        if v(self._oerp.version) < v('6.1'):
             return self.unlink([obj.id], context)
         else:
             return self.unlink([obj.id], context=context)
@@ -210,11 +227,11 @@ class Model(object):
         """Provide a dynamic access to a RPC method."""
         def rpc_method(*args, **kwargs):
             """Return the result of the RPC request."""
-            if v(self._oerp._version) < v('6.1'):
+            if v(self._oerp.version) < v('6.1'):
                 if kwargs:
                     raise error.RPCError(
-                        u"Named parameters are not supported by this version "
-                        u"of OpenERP")
+                        "Named parameters are not supported by this version "
+                        "of OpenERP")
                 result = self._oerp.execute(
                     self._browse_class.__osv__['name'], method, *args)
             else:
@@ -238,7 +255,7 @@ class Model(object):
 
     def __iter__(self):
         ids = self.search([])
-        return self._browse_generator(ids)
+        return browse.BrowseRecordIterator(self, ids)
 
     def __len__(self):
         return self._oerp.search(self._browse_class.__osv__['name'], count=True)
